/*
 * Copyright (c) 2015, The University of Sheffield. See the file COPYRIGHT.txt
 * in the software or at http://gate.ac.uk/gate/COPYRIGHT.txt
 * 
 * This file is part of GATE (see http://gate.ac.uk/), and is free software,
 * licenced under the GNU Library General Public License, Version 2, June 1991
 * (in the distribution as file licence.html, and also available at
 * http://gate.ac.uk/gate/licence.html).
 * 
 * Mark A. Greenwood 12/08/2015
 */
package gate.sentistrength;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Resource;
import gate.Utils;
import gate.creole.AbstractLanguageAnalyser;
import gate.creole.ExecutionException;
import gate.creole.ResourceInstantiationException;
import gate.creole.metadata.CreoleParameter;
import gate.creole.metadata.CreoleResource;
import gate.creole.metadata.Optional;
import gate.creole.metadata.RunTime;
import gate.util.Files;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import uk.ac.wlv.sentistrength.SentiStrength;

@CreoleResource(name = "SentiStrength", helpURL = "http://sentistrength.wlv.ac.uk/", icon = "SentiStrength", comment = "SentiStrength: Automatic Sentiment Analysis")
public class SentiStrengthPR extends AbstractLanguageAnalyser {

  private static final long serialVersionUID = -2176547670146744425L;

  private SentiStrength sentiStrength;

  private URL dataDir;

  private String annotationType, annotationSetName;

  private Boolean debug;

  private List<String> options;

  @CreoleParameter(defaultValue = "./resources/", comment = "The folder containing the SentiStrength resource files (must be a file:// URL)")
  public void setDataDir(URL dataDir) {
    this.dataDir = dataDir;
  }

  public URL getDataDir() {
    return dataDir;
  }

  public List<String> getOptions() {
    return options;
  }

  @Optional
  @CreoleParameter(defaultValue = "", comment = "Any other SentiStrength options you want to set")
  public void setOptions(List<String> options) {
    this.options = options;
  }

  public String getAnnotationType() {
    return annotationType;
  }

  @RunTime
  @CreoleParameter(defaultValue = "Sentence", comment = "The annotation type covering the text to analyse")
  public void setAnnotationType(String annotationType) {
    this.annotationType = annotationType;
  }

  public String getAnnotationSetName() {
    return annotationSetName;
  }

  @RunTime
  @Optional
  @CreoleParameter(comment = "The annotation set holding the annotations to process")
  public void setAnnotationSetName(String annotationSetName) {
    this.annotationSetName = annotationSetName;
  }

  public Boolean getDebug() {
    return debug;
  }

  @CreoleParameter(defaultValue = "false")
  public void setDebug(Boolean debug) {
    this.debug = debug;
  }

  @Override
  public Resource init() throws ResourceInstantiationException {

    try {

      // While the param is a URL SentiStrength need a file path to the
      // resources folder, even worse it has to end in a separator as it seems
      // to do a string addition rather than a relative File object creation
      // so...

      // convert the URL to a File object (will throw an exception if the URL
      // isn't a file://)
      File dataFilesDir = Files.fileFromURL(dataDir);

      // even if the URL is a File we need to throw an exception if it doesn't
      // exist or isn't a directory
      if(!dataFilesDir.exists() || !dataFilesDir.isDirectory())
        throw new ResourceInstantiationException(
          "The URL provided for the dataDir param doesn't exist or isn't a directory");

      // now we know we have a pointer to a directory convert it to a String...
      String dataFilesPath = dataFilesDir.getCanonicalPath();

      // ... and ensure it ends with a separator char so we can safely use it to
      // Initialise the SentiStrength library
      if(!dataFilesPath.endsWith(File.separator))
        dataFilesPath += File.separator;

      // create a list to hold all the config options
      List<String> config = new ArrayList<String>();

      // set the path to the resources folder
      config.add("sentidata");
      config.add(dataFilesPath);

      // we use the "scale" option when running SentiStrength
      config.add("scale");

      // if debug is on then add the "explain" option as well
      if(debug) config.add("explain");

      // if other options have been provided then add them to the end of the
      // config list as well
      if(options != null) config.addAll(options);

      // ... so finally we can create...
      sentiStrength = new SentiStrength();

      // ... and initialise the library, unfortunately if this fails we don't
      // get an exception thrown there is just a message printed out
      sentiStrength.initialise(config.toArray(new String[config.size()]));

      // and we are finally finished, yay!
      return this;
    } catch(IOException e) {
      throw new ResourceInstantiationException(
        "Unable to initialise the SentiStrength PR", e);
    }
  }

  @Override
  public void execute() throws ExecutionException {

    // the annotations that span the text we want to determine the sentiment for
    AnnotationSet annotations =
      document.getAnnotations(annotationSetName).get(annotationType);

    for(Annotation annotation : annotations) {
      // for each annotation...

      // run SentiStrength over the text and store the result
      String result =
        sentiStrength.computeSentimentScores(Utils.stringFor(document,
          annotation));

      // regardless of options the important numbers are space separated at the
      // beginning of the line so split the line on spaces
      String[] parts = result.split("\\s+");

      // we used the "scale" option so the main score is in the 3rd array
      // position once we've split the result on spaces
      int score = Integer.parseInt(parts[2]);

      // store the score and other values as integer features on the annotation
      annotation.getFeatures().put("score", score);
      annotation.getFeatures().put("positive", Integer.parseInt(parts[0]));
      annotation.getFeatures().put("negative", Integer.parseInt(parts[1]));

      // use the value of the score to give a textual polarity description
      String polarity = "neutral";
      if(score < 0)
        polarity = "negative";
      else if(score > 0) polarity = "positive";

      // store the polarity info
      annotation.getFeatures().put("polarity", polarity);

      // if we are in debug mode then store the full SentiStrength explanation
      // as a feature on the annotation
      if(debug) annotation.getFeatures().put("debug", result);
    }
  }
}
